// This file is a part of the TTRM Program;
// copyright (C) 2001-2002, Alexandros Droseltis;
// it is distributed under the terms of the
// GNU General Public Licence v.2 (read the file LICENCE)
#include"board.h"
#include"func_dialog.moc"
#include<string.h>
#include<stdlib.h>
#include<qfile.h>
enum{SchliessenID=255, SpeichernID, SpeichernUnterID};
enum{viewRowsID=300, viewVerticalsID};
enum{TEX, DVI, PS, PDF, GER, NUM, ROWS, VERT};

Board::Board(){
    initialize();

//--------------------------------------------------
// BEGINNING(File Menu - QPopupMenu)
//--------------------------------------------------
    dateiMenu = new QPopupMenu;
    dateiMenu->insertItem("&Open...", this,
			  SLOT(openSlot()), CTRL + Key_O);
    dateiMenu->insertSeparator();
    dateiMenu->insertItem( "&Save", this, SLOT(speichernSlot()),
			   CTRL + Key_S, SpeichernID );
    dateiMenu->setItemEnabled( SpeichernID, true );
    dateiMenu->insertItem( "Save as...",
			   this, SLOT( speichernUnterSlot() ),
			   0, SpeichernUnterID );
    dateiMenu->setItemEnabled( SpeichernUnterID, true );
    dateiMenu->insertSeparator();
    dateiMenu->insertItem("Export", export_files);
    export_files->insertItem("Row Matrix", ROWS);
    export_files->insertItem("Verticals", VERT);
    export_files->setItemEnabled(VERT, false);
    export_files->insertItem("German Square Matrix", GER);
    export_files->insertItem("Nummerical Square Matrix", NUM);
    connect(export_files, SIGNAL(activated(int)),
	    this, SLOT(export_tex(int)));

//     connect(export_files, SIGNAL(activated(int)),
// 	    this, SLOT(export_square(int)));

    dateiMenu->insertSeparator();
    dateiMenu->insertItem("&Quit", this,
			  SLOT(beendenSlot()), CTRL + Key_Q);

//--------------------------------------------------
// END(File Menu - QPopupMenu)
//--------------------------------------------------


//--------------------------------------------------
// BEGINING(View Menu - QPopupMenu)
//--------------------------------------------------
    viewMenu=new QPopupMenu(this);
    viewRows=new QPopupMenu(this);
    viewVerticals=new QPopupMenu(this);

    viewMenu->insertItem("Row-Matrix", viewRows);
    viewMenu->insertItem("Verticals", viewVerticals, viewVerticalsID);
    viewMenu->setItemEnabled(viewVerticalsID, false);
    viewMenu->insertItem("German Matrix", GER);
    viewMenu->insertItem("Nummerical Matrix", NUM);
    connect(viewMenu, SIGNAL(activated(int)),
	    this, SLOT(view_square(int)));

    viewRows->insertItem("DVI", DVI);
    viewRows->setAccel(CTRL + Key_D, DVI);
    viewRows->insertItem("PS", PS);
    viewRows->setAccel(CTRL+Key_P, PS);
    viewRows->insertItem("PDF", PDF);
    viewRows->setAccel(CTRL+Key_F, PDF);
    connect(viewRows, SIGNAL(activated(int)), this, SLOT(view_rows(int)));

    viewVerticals->insertItem("DVI", DVI);
    viewVerticals->insertItem("PS", PS);
    viewVerticals->insertItem("PDF", PDF);
    connect(viewVerticals, SIGNAL(activated(int)), this, SLOT(view_vert(int)));

//--------------------------------------------------
// END(View Menu - QPopupMenu)
//--------------------------------------------------

    helpMenu=new QPopupMenu(this);
    helpMenu->insertItem("Manual", this, SLOT(help_slot()), Key_F1);
    helpMenu->insertItem("About TTRM", this, SLOT(about_slot()));

    my_menu=new QMenuBar(this);
    my_menu->insertItem("&File", dateiMenu);
    my_menu->insertItem("&View", viewMenu);
    my_menu->insertItem("&Help/Info", helpMenu);

    QLabel *row_label=new QLabel(this);
    row_label->setText("Row:");
    row_label->setGeometry(10, 30, 40, 25);
    row_edit=new QLineEdit(this);
    row_edit->setGeometry(50, 30, 350, 25);

    QLabel *title_label=new QLabel(this);
    title_label->setText("Title:");
    title_label->setGeometry(10, 80, 40, 25);
    title_edit=new QLineEdit(this);
    title_edit->setGeometry(50, 80, 230, 25);
     QObject::connect(row_edit, SIGNAL(textChanged(const QString &)),
 		     this, SLOT(send_row()));
     QObject::connect(title_edit, SIGNAL(textChanged(const QString & )),
 		     this, SLOT(send_title()));

    my_dialog=new func_dialog(this);
    my_dialog->setGeometry(10, 130, 400, 560);
     QObject::connect(my_dialog, SIGNAL(v_clicked()),
 		     this, SLOT(vert_enable()));
}

bool Board::row_is_valid(void){
    int i, j;
    QStringList qsep_notes;
    QString notes[]={"his", "c", "deses",
		     "hisis", "cis", "des",
		     "cisis", "d", "eses",
		     "dis", "es", "feses",
		     "disis", "e", "fes",
		     "eis", "f", "geses",
		     "eisis", "fis", "ges",
		     "fisis", "g", "ases",
		     "gis", "as", "as",
		     "gisis", "a", "heses",
		     "ais", "b", "ceses",
		     "aisis", "h", "ces"};

    QMessageBox *not_valid=new QMessageBox(this);
    not_valid->setCaption("ttrm: row control");
    not_valid->setText("The row is not valid!");
    not_valid->setIcon(QMessageBox::Warning);

    qsep_notes=QStringList::split(" ", row_edit->text());
    i=0;
    while(qsep_notes[i]!=NULL) i++;
    int num_elem=i;
    plithos=i;
    if(!plithos){
	not_valid->show();
	return false;
    }
     for(i=0; i<num_elem; i++){
 	 qsep_notes[i].remove(qsep_notes[i].find(QRegExp("[0-9]")),
			      qsep_notes[i].find(QRegExp("[0-9]")));
	 for(j=0; j<36; j++){
	     if(qsep_notes[i]==notes[j]) break;
	 }
	 if(j==36){
	     not_valid->show();
	     return false;
	 }
     }
     return true;
}

void Board::view_rows(int file_type){
    if(row_is_valid()){
	set_options(my_dialog->get_opt());
	opt[11]=0;
	calculate();
	system("cd /tmp/; tex newfile.tex");
	switch(file_type){
	    case DVI:
		system("xdvi /tmp/newfile.dvi&");
		break;
	    case PS:
		system("cd /tmp/; dvips -o newfile.ps newfile.dvi");
		system("gv -a4 -antialias /tmp/newfile.ps&");
		break;
	    case PDF:
		system("cd /tmp/; dvipdf newfile.dvi newfile.pdf");
		system("gv -a4 -antialias /tmp/newfile.pdf&");
		break;
	}
    }
}

void Board::export_tex(int table_type){
    if(row_is_valid()){
//
// Take the name AND format from the filedialog.
//
	QFileDialog *export_dialog=new QFileDialog(this);
	QString name, command, the_filters;
	export_dialog->setMode(QFileDialog::AnyFile);
	set_options(my_dialog->get_opt());
	switch(table_type){
	    case ROWS:
		opt[11]=0;
		export_dialog->setCaption("ttrm: Export Row Matrix");
		the_filters="*.tex;;*.dvi;;*.ps;;*.pdf";
		break;
	    case VERT:
		opt[10]=0;
		export_dialog->setCaption("ttrm: Export Verticals");
		the_filters="*.tex;;*.dvi;;*.ps;;*.pdf";
		break;
	    case GER:
		opt[8]=0;
		export_dialog->setCaption("ttrm: Export German Square");
		the_filters="*.txt";// TO DO: *.tex (latex)
		break;
	    case NUM:
		opt[9]=0;
		export_dialog->setCaption("ttrm: Export Nummerical Square");
		the_filters="*.txt";// TO DO: *.tex (latex)
		break;
	}
    
	export_dialog->setFilters(the_filters);
	if(export_dialog->exec()==QDialog::Accepted){
	    name=export_dialog->selectedFile();
//
// Add the ending, if needed.
//
	    QString ending1, ending2;
	    ending1=export_dialog->selectedFilter();
	    ending2=export_dialog->selectedFilter();
	    ending1.remove(0, 1);// e.g. .dvi
	    ending2.remove(0, 2);// e.g. dvi

	    if(name.section('.', -1, -1)!=ending2)
		name+=ending1;
//
// Print to file.
//

	    if(table_type==ROWS || table_type==VERT){
		if(export_dialog->selectedFilter()==QString("*.tex")){
		    calculate(export_dialog->selectedFile().latin1());
		}
		else{
		    calculate();
		    system("cd /tmp/;tex newfile.tex");
		    if(export_dialog->selectedFilter()==QString("*.dvi")){
			command="mv /tmp/newfile.dvi "+name;
			system(command.latin1());
		    }
		    else if(export_dialog->selectedFilter()==QString("*.ps")){
			system("cd /tmp/; dvips -o newfile.ps newfile.dvi");
			command="mv /tmp/newfile.ps "+name;
			system(command.latin1());
		    }
		    else if(export_dialog->selectedFilter()==QString("*.pdf")){
			system("cd /tmp/; dvipdf newfile.dvi newfile.pdf");
			command="mv /tmp/newfile.pdf "+name;
			system(command.latin1());
		    }
		}
	    }
	    else if(table_type==GER || table_type==NUM){
		calculate(export_dialog->selectedFile().latin1());
	    }
	}
    }
}

void Board::view_vert(int file_type){
    if(row_is_valid()){
	set_options(my_dialog->get_opt());
	opt[10]=0;
	calculate();
	system("cd /tmp/; tex newfile.tex");
	switch(file_type){
	    case DVI:
		system("xdvi /tmp/newfile.dvi&");
		break;
	    case PS:
		system("cd /tmp/; dvips -o newfile.ps newfile.dvi");
		system("gv -a4 -antialias /tmp/newfile.ps&");
		break;
	    case PDF:
		system("cd /tmp/; dvipdf newfile.dvi newfile.pdf");
		system("gv -a4 -antialias /tmp/newfile.pdf&");
		break;
	}
    }
}

void Board::view_square(int table_type){
    if(row_is_valid()){// defines also the $plithos
	set_options(my_dialog->get_opt());
	if(table_type==GER) opt[12]=0;
	else if(table_type==NUM) opt[13]=0;
	calculate();
	QDialog *square_view=new QDialog(this);
	QTextEdit *square_content=new QTextEdit(square_view);
	square_content->setGeometry(0, 0, 50*plithos, 20*plithos);
	square_content->setReadOnly(true);
	square_content->setText(square);
	square_view->show();
    }
}

void Board::initialize(void){
    for(int i=0; i<24; i++) opt[i]=1;
    strcpy(head, "");
    strcpy(line_row, "");
    strcpy(title, "");
    export_files=new QPopupMenu(this);
}

void Board::openSlot(void){
    QString name = QFileDialog::getOpenFileName(".", "*.ttrm", this);
    if (!name.isEmpty()){
	dateiName = name;
	leseDatei();
 	dateiMenu->setItemEnabled(SpeichernID, true);
 	dateiMenu->setItemEnabled(SpeichernUnterID, true);
 	QString s1=name.section('.', 0, 0);
 	QString s2=s1.section('/', -1, -1);
//	send_file_head(s2);
    }
}

// void Board::send_file_head(QString str){
//     my_dialog->set_head(str);
// }

void Board::beendenSlot(){
    system("rm -f /tmp/newfile.tex /tmp/newfile.log /tmp/newfile.dvi /tmp/newfile.ps /tmp/newfile.pdf /tmp/newfile_num.txt /tmp/newfile_ger.txt");
    qApp->quit();
}

void Board::leseDatei(){
    infile=new ifstream(dateiName);
    infile->getline(line_row, 100);
    infile->getline(title, 100);
    infile->close();
    QString str1(line_row);
    QString str2(title);
    row_edit->setText(str1);
    title_edit->setText(str2);
}

void Board::speichernSlot(){
    speichereDatei(false);
}

void Board::speichernUnterSlot(){
    speichereDatei(true);
}

void Board::speichereDatei(bool neuerName){
    QFileDialog *select=new QFileDialog(this);
    QMessageBox *exists=
	new QMessageBox("TTRM Save as",
			"The file exists! Overwrite?",
			QMessageBox::Warning,
			QMessageBox::Yes,
			QMessageBox::No,
			QMessageBox::Cancel|QMessageBox::Escape,
			this);

    int x=1;
    if (dateiName.isEmpty() || neuerName == true){
  	while(x){
	    dateiName=select->getSaveFileName(".", "*.ttrm", this);
	    if(!dateiName.isEmpty()){
		if(dateiName.section('.', -1, -1)!="ttrm")
		    dateiName+=".ttrm";
		if(QFile::exists(dateiName)){
		    switch(exists->exec()){
			case QMessageBox::Yes: 
			    write2file(dateiName);
			    x=0;
			    break;
			case QMessageBox::Cancel:
			    x=0;
			    break;
			case QMessageBox::No:
			    break;
		    }
		}
		else{
		    write2file(dateiName);
		    break;
		}
	    }
	    else break;
	}
    }
    else write2file(dateiName);
}

void Board::write2file(QString name){
    outfile=new ofstream(dateiName);
    *outfile << row_edit->text() << endl;
    *outfile << title_edit->text() << endl;
    outfile->close();
    QString s1=dateiName.section('.', 0, 0);
    QString s2=s1.section('/', -1, -1);
//    send_file_head(s2);
}

void Board::vert_enable(void){
    viewMenu->setItemEnabled(viewVerticalsID, my_dialog->get_checked_v());
    export_files->setItemEnabled(VERT, my_dialog->get_checked_v());
}

bool* Board::get_opt(){
   return opt;
}

int Board::get_opt(int i){
   return opt[i];
}

void Board::send_row(void){
    my_dialog->send_row(row_edit->text());
}

void Board::send_title(void){
    my_dialog->send_title(title_edit->text());
}

char* Board::get_row(void){
    return line_row;
}

char* Board::get_title(void){
    return title;
}
 
// const char* Board::get_head(void){
//     return head;
// }

void Board::set_options(bool* opt){
    int i;
    for(i=0; i<8; i++)
	this->opt[i]=opt[i];
    for(i=8; i<=13; i++)
	this->opt[i]=1;
}

void Board::calculate(const char* str="/tmp/newfile"){
    myttrm=new ttrm(row_edit->text().latin1(),
		    title_edit->text().latin1(),
		    opt,
		    str,
		    my_dialog->get_form(),
		    my_dialog->get_trans());
    myttrm->exec(opt);
    square=QString(myttrm->pack(opt));
}

void Board::make_file_names(void){
    QString head0(head);
    if(opt[11]==0) head1=head0+".tex";
    else head1="";
    if(opt[10]==0) head2=head0+"_vert.tex";
    else head2="";
    if(opt[9]==0) head3=head0+"_num.txt";
    else head3="";
    if(opt[8]==0) head4=head0+"_ger.txt";
    else head4="";
}

void Board::help_slot(){
    int i;
    QTextBrowser *manual=new QTextBrowser();
    manual->setGeometry(500, 100, 500, 800);
    manual->setText("#include<manual.html>");
    manual->show();
}

void Board::about_slot(){
    about=new QTabDialog(this);
    about->setCaption("About TTRM");

    QLabel *apropos=new QLabel(about);
    QString str1;
    str1="A generator of several musical arrays\n";
    str1+="and a search maschine for sets in row matrices\n";
    str1+=" 2001-2002 Alexandros Droseltis";
    str1+="Made with C++/QT3";
    apropos->setText(str1);
    apropos->setAlignment(Qt::AlignCenter);

    QLabel *author=new QLabel(about);
    QString str2;
    str2="Alexandros Droseltis\n";
    str2+="rakewell@udk-berlin.de";
    author->setText(str2);
    author->setAlignment(Qt::AlignCenter);

    QLabel *thanks=new QLabel(about);
    QString str3;
    str3="* Christian Mondrup for testing ttrm\n  and for his creative advices";
    thanks->setText(str3);

    gpl=new QTextEdit(about);
    gpl->setText("#include<LICENCE>");
    gpl->setReadOnly(true);

    about->setGeometry(300, 200, 300, 300);
    about->addTab(apropos, "apropos");
    about->addTab(author, "Author");
    about->addTab(thanks, "Thanks");
    about->addTab(gpl, "Licence");
    about->show();
}
