// This file is a part of the TTRM Program;
// copyright (C) 2001-2002, Alexandros Droseltis;
// it is distributed under the terms of the
// GNU General Public Licence v.2 (read the file LICENCE)
#include"note.h"
//  Note::Note(int initialAccid, int initialHead){
//    itsAccid=initialAccid;
//    itsHead=initialHead;
//  }

//  Note::~Note(){
//  }

// "oct_d_trans" transposes the notehead "$oct" octaves downwards.
void Note::oct_d_trans(int oct){
  itsHead+=(-7)*oct;
}

const int Note::getAccid(void){
  return itsAccid;
}

const int Note::getHead(void){
  return itsHead;
}

void Note::setAccid(int Accid){
  itsAccid=Accid;
}

void Note::setHead(int Head){
  itsHead=Head;
}

// enharmonic changes the note in its enharmonic one.
void Note::enharmonic(){
  // If the accidental is a sharp...
  if(itsAccid>0){
    // and the note is neither B nor E:
    itsAccid+=(itsHead%7!=6 && itsHead%7!=2) ? -2 : -1;
    // The former line checks $itsHead; the latter
    // is being changed at the next line!
    itsHead++;
  }
  // If the accidental is a flat...
  else if(itsAccid<0){
    // ...and the note is neither C nor F:
    itsAccid+=(itsHead%7!=0 && itsHead%7!=3) ? 2 : 1;
    itsHead--;
  }
  // If the accidental is a natural...
  else{
    // ...and the note is a C or a F:
    if(itsHead%7==0 || itsHead%7==3){
      itsHead--;
      itsAccid=1;
    }
    // ...and the note is a B or E:
    if(itsHead%7==6 || itsHead%7){
      itsHead++;
      itsAccid=-1;
    }
  }
}

// first_step reads the quantity and the quality of the transposition
// interval and sends a number to the function "transpositive". This
// number (*classi) enables the function transpositive to make a right
// transposition of the accidental according to the quality of the
// transposition interval, by keeping for every interval the same
// algorithm.

// INPUT: int interval (interval quantity, e.g. 4=fourth, 5=fifth),
// char type (interval quality, e.g. augmented, minor).
 
// OUTPUT: The number that is beiing sent to "transpositive". It
// defines the transposition of the accidental for each case.

int Note::first_step(Interval Diastima){
  int interval=Diastima.get_space();
  int type=Diastima.get_type();
  // Fourth and Octave
  if((interval+7)%7==4 || (interval+7)%7==1){
      if(type=='d') return -2;
      else if(type=='p') return -1;
      else if(type=='a') return 0;
  }
  // Fifth
  else if((interval+7)%7==5){
    if(type=='d') return -1;
    else if(type=='p') return 0;
    else if(type=='a') return 1;
  }
  // 2nds, 3rds, 6ths, 7ths
  else{
    if(type=='d') return -2;
    else if(type=='a') return 1;
    else if(type=='m') return -1;
    else if(type=='M') return 0;
    else{
      return -1;
      //cout << "Please check your entry and try again" << endl;
    }
  }
}

void Note::transpositive(Interval Diastima){
  itsAccid+=Tr_Condition(Diastima.get_space())+first_step(Diastima);
  // Attention: Tr_Condition checks the itsHead; the latter is
  // being changed in the next line!
  itsHead+=Diastima.get_space()-1;
}

// if Tr_condition=0 => minor accidental transposition
// if Tr_condition=1 => major accidental transposition

// What is "major" and "minor" accidental transposition?

// If a note is being transposed by a certain interval, there are two
// cases:

// a) the transposed note has an accidental of value x;
// b) the transposed note has an accidental of value x+1;

// INPUT: int interval

// OUTPUT
// O or 1 (s. "transpositive")

// Return 0, if the accidental transposition is minor:
// INTERVAL HEAD
// 2nd      !(e, b)
// 3rd      c, f, g
// 4th      f
// 5th      !b
// 6th      !(e, a, b)
// 7th      c, f

// Return 1, if the accidental transposition is major (the rest of the
// cases)

int Note::Tr_Condition(int space){
  if(((space+7)%7==2 && ((itsHead+7)%7!=2 && (itsHead+7)%7!=6)) || 
     ((space+7)%7==3 && ((itsHead+7)%7==0 || (itsHead+7)%7==3 || (itsHead+7)%7==4)) ||
     ((space+7)%7==6 && ((itsHead+7)%7!=2 && (itsHead+7)%7!=5 && (itsHead+7)%7!=6)) ||
     ((space+7)%7==0 && ((itsHead+7)%7==0 || (itsHead+7)%7==3)) ||
     ((space+7)%7==4 && (itsHead+7)%7==3) ||
     ((space+7)%7==5 && (itsHead+7)%7!=6))
    return (0);
  else return(1);
}

// "negative" returns the complementary (not regards to the number of
// the octaves that it contains) of an interval. For example, if the
// source interval is -7M, the destination interval will be +2m, if
// the source interval is -10m, the destination interval will be +3M.
// "negative" remembers how many octaves upwards is the interval
// transformed, so that the function "oct_d_trans" will correct later
// the transposed note to its right octave place.

// INPUT: int interval1 (the quantity of the transposition interval),
// char class1 (the quality of the transposition interval).

// OUTPUT: int *interval (the complemetary interval --not regards to
// the octaves it contains), int *class2 (the type of the complemetary
// interval) and int *octave (the number of the octaves it was upwards
// transformed).

void Note::negative(Interval Diastima1, Interval &Diastima2, int *octave)
{
  int i;
  char quality[]={'d', 'm', 'p', 'M', 'a'};
  i=0;
  Diastima2.set_space((Diastima1.get_space()+2)%7+7);
  *octave=((-1)*Diastima1.get_space()-2)/7+1;

  for(i=0; i<5; i++){
    if(Diastima1.get_type()==quality[i]){
      Diastima2.set_type(quality[4-i]);
      break;
    }
  }
}

// "transpose" transposes a note according to a given interval.

// If the intervals is upward, the "transupper" makes the
// transposition. If the interval is downward, "negative" gives an
// alternative interval for the transposition, the complementary one
// (not regards to the number of octaves that are contained in the
// transposition interval)."negative" also finds out the number of
// octaves are needed in order to transform the transposition. It
// sends the alternative interval (it is upward) to the "transpose"
// that transposes the note upwards according to it. Then,
// "oct_d_trans" transposes the transposed note as many octaves
// downwards as "negative" needed to transform the given transposition
// interval.

// For example, if the transposition interval is -9m, this procces is the 
// following:

// a) "negative" returns the alternative transposition interval 7M,
// the number of the octaves that have been needed for the
// transformation, i.d. 2 (-9m => -2m => 7m).

// b) "transupper" transposes the note a 7M upwards.

// c) "oct_d_trans" transposes the transposed note 2 octaves
// downwards.

void Note::transpose(Interval Diastima){
  Interval Diastima2;
  int octave;

  if(Diastima.get_space()>0) transpositive(Diastima);
  else{
      negative(Diastima, Diastima2, &octave);
      transpositive(Diastima2);
      oct_d_trans(octave);
  }
}

// The function "notename2num" changes the names of the noteheads
// into numbers according to the following scheme:
void Note::tex2num(){
  int k=1, i;
  char accid[]={'<', '_', '=', '^', '>'};
  for(i=0; i<5; i++){
    if(TeXNote[0]==accid[i]){
      itsAccid=i-2;
      break;
    }
  }
  if(i==5){
    itsAccid=0;
    k=0;
  }
  char heads[]={
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
    'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'};
  for(i=0; i<40; i++)
    if(TeXNote[k]==heads[i]) itsHead=i+5;
}

int Note::basic_german(char x){
  switch(x){
  case 'c': return 7;
  case 'd': return 8;
  case 'e': return 9;
  case 'f': return 10;
  case 'g': return 11;
  case 'a': return 12;
  case 'h': return 13;
  default: return 13;
  }
}

int Note::octave_german(char x){
  switch(x){
  case '1': return 2;
  case '2': return 3;
  case '3': return 4;
  default: return 1;
  }
}


int Note::accid2num_german(char x[])
{
  /* If the name of the note is Nx or N (N is a name of a notehead,
     and x the octave number)... */
  if(isdigit(x[1])!=0 || x[1]=='\0'){
    /* if the name of the notehead is NOT "b", the accidental is natural;
       else (if the name of the notehead IS "b"), the accidental is flat. */
    if(x[0]!='b') return 0;
    else return -1;
  }
  else{
    /* SHARP, DOUBLE SHARP */
    if(x[1]=='i' && x[2]=='s'){
      if(isdigit(x[3])!=0 || x[3]=='\0') return 1;
      else if(x[3]=='i' && x[4]=='s') return 2;
    }
    /* FLAT, DOUBLE FLAT */
    /* if the notehead is NOT an "e", "a" or "b flat": */
    else if(x[0]!='e' && x[0]!='a' && x[0]!='b'){
      /* if the accidental is a flat: */
      if(x[1]=='e' && x[2]=='s'){
	/* if it is NOT double flat: */
	if(isdigit(x[3])!=0 || x[3]=='\0') return -1;
	/* if it IS double flat: */
	else if(x[3]=='e' && x[4]=='s') return -2;
      }
    }
    /* if the notehead IS an "e", "a": */
    else if(x[0]=='e' || x[0]=='a'){
      /* if the accidental is a flat: */
      if(x[1]=='s'){
	/* if it is NOT double flat: */
	if(isdigit(x[2])!=0 || x[2]=='\0') return -1;
	/* if it IS double flat: */
	else if(x[2]=='e' && x[3]=='s') return -2;
      }
    }
    /* if the name of the notehead IS a "b flat": */
    else if(x[0]=='b'){
      /* if it has NOT a flat ending (-es): */
      if(isdigit(x[1])!=0 || x[1]=='\0') return -1;
      /* if it DOES have a flat ending (-es): */
      else if(x[2]=='e' && x[3]=='s') return -2;
    }
  }
}

void Note::german2num(void){
  int occupied=strlen(gNote);
  itsHead=basic_german(gNote[0])+octave_german(gNote[occupied-1])*7;
  itsAccid=accid2num_german(gNote);
}

void Note::set_gNote(char g_note[]){
  strcpy(gNote, g_note);
}

void Note::set_TeXNote(char tex_note[]){
  strcpy(TeXNote, tex_note);
}
